@@ -17,12 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "SubtreeWatcher.hh"
 #include "../mediascanner/MediaStore.hh"
 #include "../mediascanner/MediaFile.hh"
 #include "InvalidationSender.hh"
 #include "MetadataExtractor.hh"
-#include "SubtreeWatcher.hh"
-#include "util.h"
+#include "../mediascanner/internal/utils.hh"
 
 #include<sys/select.h>
 #include<stdexcept>
@@ -101,6 +101,11 @@ void SubtreeWatcher::addDir(const string &root) {
                 root.c_str(), __PRETTY_FUNCTION__);
         return;
     }
+    if(has_scanblock(root)) {
+        fprintf(stderr, "Directory %s has a scan block file, skipping it.\n",
+                root.c_str());
+        return;
+    }
     if(p->str2wd.find(root) != p->str2wd.end())
         return;
     unique_ptr<DIR, int(*)(DIR*)> dir(opendir(root.c_str()), closedir);
@@ -121,7 +126,7 @@ void SubtreeWatcher::addDir(const string &root) {
     unique_ptr<struct dirent, void(*)(void*)> entry((dirent*)malloc(sizeof(dirent) + NAME_MAX),
                     free);
     struct dirent *de;
-        while(readdir_r(dir.get(), entry.get(), &de) == 0 && de ) {
+    while(readdir_r(dir.get(), entry.get(), &de) == 0 && de ) {
         struct stat statbuf;
         string fname = entry.get()->d_name;
         if(fname[0] == '.') // Ignore hidden entries and also "." and "..".
@@ -152,8 +157,15 @@ void SubtreeWatcher::fileAdded(const string &abspath) {
     printf("New file was created: %s.\n", abspath.c_str());
     try {
         DetectedFile d = p->extractor.detect(abspath);
-        // Only extract and insert the file if the ETag has changed.
-        if (d.etag != p->store.getETag(d.filename)) {
+        if(p->store.is_broken_file(abspath, d.etag)) {
+            fprintf(stderr, "Using fallback data for unscannable file %s.\n", abspath.c_str());
+            p->store.insert(p->extractor.fallback_extract(d));
+        } else if (d.etag != p->store.getETag(d.filename)) {
+            // Only extract and insert the file if the ETag has changed.
+            p->store.insert_broken_file(abspath, d.etag);
+            // If detection dies, insertion into broken files persists
+            // and the next time this file is encountered, it is skipped.
+            // Insert cleans broken status of the file.
             p->store.insert(p->extractor.extract(d));
         }
     } catch(const exception &e) {
